@@ -16,6 +16,7 @@ from . import types
 from .. import const
 from ..const import COMPOSEFILE_V1 as V1
 from ..utils import build_string_dict
+from ..utils import parse_bytes
 from ..utils import parse_nanoseconds_int
 from ..utils import splitdrive
 from ..version import ComposeVersion
@@ -108,6 +109,7 @@ DOCKER_CONFIG_KEYS = [
 ]
 
 ALLOWED_KEYS = DOCKER_CONFIG_KEYS + [
+    'blkio_config',
     'build',
     'container_name',
     'credential_spec',
@@ -402,11 +404,12 @@ def load_mapping(config_files, get_func, entity_type, working_dir=None):
 
             external = config.get('external')
             if external:
+                name_field = 'name' if entity_type == 'Volume' else 'external_name'
                 validate_external(entity_type, name, config)
                 if isinstance(external, dict):
-                    config['external_name'] = external.get('name')
-                else:
-                    config['external_name'] = name
+                    config[name_field] = external.get('name')
+                elif not config.get('name'):
+                    config[name_field] = name
 
             if 'driver_opts' in config:
                 config['driver_opts'] = build_string_dict(
@@ -571,7 +574,7 @@ class ServiceExtendsResolver(object):
         config_path = self.get_extended_config_path(extends)
         service_name = extends['service']
 
-        if config_path == self.service_config.filename:
+        if config_path == self.config_file.filename:
             try:
                 service_config = self.config_file.get_service(service_name)
             except KeyError:
@@ -714,9 +717,6 @@ def process_service(service_config):
     if 'labels' in service_dict:
         service_dict['labels'] = parse_labels(service_dict['labels'])
 
-    if 'extra_hosts' in service_dict:
-        service_dict['extra_hosts'] = parse_extra_hosts(service_dict['extra_hosts'])
-
     if 'sysctls' in service_dict:
         service_dict['sysctls'] = build_string_dict(parse_sysctls(service_dict['sysctls']))
 
@@ -726,8 +726,9 @@ def process_service(service_config):
         if field in service_dict:
             service_dict[field] = to_list(service_dict[field])
 
-    service_dict = process_healthcheck(service_dict, service_config.name)
-    service_dict = process_ports(service_dict)
+    service_dict = process_blkio_config(process_ports(
+        process_healthcheck(service_dict, service_config.name)
+    ))
 
     return service_dict
 
@@ -754,6 +755,28 @@ def process_depends_on(service_dict):
     return service_dict
 
 
+def process_blkio_config(service_dict):
+    if not service_dict.get('blkio_config'):
+        return service_dict
+
+    for field in ['device_read_bps', 'device_write_bps']:
+        if field in service_dict['blkio_config']:
+            for v in service_dict['blkio_config'].get(field, []):
+                v['rate'] = parse_bytes(v.get('rate', 0))
+
+    for field in ['device_read_iops', 'device_write_iops']:
+        if field in service_dict['blkio_config']:
+            for v in service_dict['blkio_config'].get(field, []):
+                try:
+                    v['rate'] = int(v.get('rate', 0))
+                except ValueError:
+                    raise ConfigurationError(
+                        'Invalid IOPS value: "{}". Must be a positive integer.'.format(v.get('rate'))
+                    )
+
+    return service_dict
+
+
 def process_healthcheck(service_dict, service_name):
     if 'healthcheck' not in service_dict:
         return service_dict
@@ -771,16 +794,12 @@ def process_healthcheck(service_dict, service_name):
     elif 'test' in raw:
         hc['test'] = raw['test']
 
-    if 'interval' in raw:
-        if not isinstance(raw['interval'], six.integer_types):
-            hc['interval'] = parse_nanoseconds_int(raw['interval'])
-        else:  # Conversion has been done previously
-            hc['interval'] = raw['interval']
-    if 'timeout' in raw:
-        if not isinstance(raw['timeout'], six.integer_types):
-            hc['timeout'] = parse_nanoseconds_int(raw['timeout'])
-        else:  # Conversion has been done previously
-            hc['timeout'] = raw['timeout']
+    for field in ['interval', 'timeout', 'start_period']:
+        if field in raw:
+            if not isinstance(raw[field], six.integer_types):
+                hc[field] = parse_nanoseconds_int(raw[field])
+            else:  # Conversion has been done previously
+                hc[field] = raw[field]
     if 'retries' in raw:
         hc['retries'] = raw['retries']
 
@@ -925,6 +944,7 @@ def merge_service_dicts(base, override, version):
     md.merge_sequence('secrets', types.ServiceSecret.parse)
     md.merge_sequence('configs', types.ServiceConfig.parse)
     md.merge_mapping('deploy', parse_deploy)
+    md.merge_mapping('extra_hosts', parse_extra_hosts)
 
     for field in ['volumes', 'devices']:
         md.merge_field(field, merge_path_mappings)
@@ -940,6 +960,8 @@ def merge_service_dicts(base, override, version):
 
     md.merge_field('logging', merge_logging, default={})
     merge_ports(md, base, override)
+    md.merge_field('blkio_config', merge_blkio_config, default={})
+    md.merge_field('healthcheck', merge_healthchecks, default={})
 
     for field in set(ALLOWED_KEYS) - set(md):
         md.merge_scalar(field)
@@ -958,6 +980,14 @@ def merge_unique_items_lists(base, override):
     return sorted(set().union(base, override))
 
 
+def merge_healthchecks(base, override):
+    if override.get('disabled') is True:
+        return override
+    result = base.copy()
+    result.update(override)
+    return result
+
+
 def merge_ports(md, base, override):
     def parse_sequence_func(seq):
         acc = []
@@ -986,19 +1016,40 @@ def merge_build(output, base, override):
     md.merge_scalar('context')
     md.merge_scalar('dockerfile')
     md.merge_scalar('network')
+    md.merge_scalar('target')
     md.merge_mapping('args', parse_build_arguments)
     md.merge_field('cache_from', merge_unique_items_lists, default=[])
     md.merge_mapping('labels', parse_labels)
     return dict(md)
 
 
+def merge_blkio_config(base, override):
+    md = MergeDict(base, override)
+    md.merge_scalar('weight')
+
+    def merge_blkio_limits(base, override):
+        index = dict((b['path'], b) for b in base)
+        for o in override:
+            index[o['path']] = o
+
+        return sorted(list(index.values()), key=lambda x: x['path'])
+
+    for field in [
+            "device_read_bps", "device_read_iops", "device_write_bps",
+            "device_write_iops", "weight_device",
+    ]:
+        md.merge_field(field, merge_blkio_limits, default=[])
+
+    return dict(md)
+
+
 def merge_logging(base, override):
     md = MergeDict(base, override)
     md.merge_scalar('driver')
     if md.get('driver') == base.get('driver') or base.get('driver') is None:
         md.merge_mapping('options', lambda m: m or {})
-    else:
-        md['options'] = override.get('options')
+    elif override.get('options'):
+        md['options'] = override.get('options', {})
     return dict(md)
 
 
