@@ -9,8 +9,12 @@ import logging
 import ntpath
 
 import six
+from docker.errors import DockerException
+from docker.utils import parse_bytes as sdk_parse_bytes
 
+from .config.errors import ConfigurationError
 from .errors import StreamParseError
+from .timeparse import MULTIPLIERS
 from .timeparse import timeparse
 
 
@@ -109,7 +113,7 @@ def microseconds_from_time_nano(time_nano):
 
 
 def nanoseconds_from_time_seconds(time_seconds):
-    return time_seconds * 1000000000
+    return int(time_seconds / MULTIPLIERS['nano'])
 
 
 def parse_seconds_float(value):
@@ -120,7 +124,7 @@ def parse_nanoseconds_int(value):
     parsed = timeparse(value or '')
     if parsed is None:
         return None
-    return int(parsed * 1000000000)
+    return nanoseconds_from_time_seconds(parsed)
 
 
 def build_string_dict(source_dict):
@@ -133,3 +137,10 @@ def splitdrive(path):
     if path[0] in ['.', '\\', '/', '~']:
         return ('', path)
     return ntpath.splitdrive(path)
+
+
+def parse_bytes(n):
+    try:
+        return sdk_parse_bytes(n)
+    except DockerException:
+        raise ConfigurationError('Invalid format for bytes value: {}'.format(n))
